# Copyright (C) 2025  Ruby-GNOME Project Team
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

module Gsk
  class Loader < GObjectIntrospection::Loader
    register_constant_rename_map("2D", "TWO_DIMENSIONAL")
    register_constant_rename_map("2D_AFFINE", "TWO_DIMENSIONAL_AFFINE")
    register_constant_rename_map("2D_TRANSLATE", "TWO_DIMENSIONAL_TRANSLATE")
    register_constant_rename_map("3D", "THREE_DIMENSIONAL")

    def load
      self.version = "4.0"
      super("Gsk")
    end

    private
    def post_load(repository, namespace)
      require_relative "rounded-rect"
    end

    def field_name(field_info, klass)
      case klass.name
      when "Gsk::RoundedRect"
        if field_info.name == "corner"
          "corners"
        else
          super
        end
      else
        super
      end
    end
  end
end
